//******************************************************************************
//******************************************************************************
/**
 * @file    BPNGDefines.h
 * @author  Markus van Pinxteren
 * @date    12.05.2010
 *
 * @brief   Defines for blue PiraT 2 Library
*/
// $Id$
//******************************************************************************
//******************************************************************************
#pragma once
#ifndef __BPNGDEFINES_H__
#define __BPNGDEFINES_H__

#ifdef _WINDOWS
#ifdef BPNGCLIENTLIB_EXPORTS
#define DECLDIR __declspec(dllexport)
#else
#define DECLDIR __declspec(dllimport)
#endif
#include <windows.h>
#include <stdint.h>
#else //not _WINDOWS
#define WINAPI
#define DECLDIR
#define VOID void
#endif //_WINDOWS

#include "stdint.h"

#ifndef _WINDOWS
#include "stdio.h"
#define BOOL bool
#endif

#ifdef _WINDOWS
static const char* BPNG_CLIENT_LIB_VERSION = "2.0.4";
static const char* BPNG_STREAMING_LIB_VERSION = "1.1.2";
#else //not _WINDOWS
static const char BPNG_CLIENT_LIB_VERSION[] = "2.0.3";
static const char BPNG_STREAMING_LIB_VERSION[] = "1.1.2";
#endif //_WINDOWS

// ===========================================================================
/// enum Error codes
/**
  An error is identified by one of the following error codes. Additional
  information may be found in the BPNGError::msg field (e.g. file path that causes
  a BPNG_LOCAL_PATH_NOT_FOUND error)
*/
enum BPNGErrCode
{
    BPNG_NOERR = 0, ///< no error

    // connection errors
    BPNG_LOGGER_NOT_FOUND = 1,      ///< The IP address the lib wanted to connect was not found
    BPNG_NOT_CONNECTED = 2,         ///< A function call failed because the logger was not connected.
    BPNG_CONNECT_FTP_FAILED = 3,    ///< Establishing the ftp connection failed
    BPNG_CONNECT_TMPBUS_FAILED = 4, ///< Establishing the TMP (Telemotive Protocol) bus connection failed
    BPNG_TMPBUS_NOT_CONNECTED = 5,  ///< TMP bus is not connected
    BPNG_AMBIGUOUS_IP = 66,         ///< multiple devices with same IP available
    BPNG_FAILED_TO_CONNECT_STREAMING = 67, ///< Streaming feature could not be connected

    // FTP errors
    BPNG_FTP_NOT_CONNECTED = 6,            ///< FTP is not connected
    BPNG_FTP_SERVER_NOT_FOUND = 7,         ///< FTP server is not fould
    BPNG_FTP_LOGIN_FAILED = 8,             ///< FTP login failed
    BPNG_FTP_REMOTE_PATH_NOT_FOUND = 9,    ///< A requested path on the FTP server is not found
    BPNG_FTP_READ_REMOTE_FILE_ERROR = 10,   ///< Can't red a file on the FTP server
    BPNG_FTP_WRITE_REMOTE_FILE_ERROR = 11,  ///< Can't write a file on the FTP server
    BPNG_FTP_TRANSFER_USER_CANCELED = 12,   ///< FTP file transfer was canceled by the user
    BPNG_FTP_CREATE_REMOTE_DIR_ERROR = 13,  ///< Can't create the directory on the FTP server
    BPNG_FTP_REMOVE_REMOTE_DIR_ERROR = 14,  ///< Can't remove the directory on the FTP server
    BPNG_FTP_REMOVE_REMOTE_FILE_ERROR = 15, ///< Can't remove the file on the FTP server
    BPNG_FTP_CHANGE_CWD_ERROR = 16,         ///< Can't change the current working directory on the FTP server

    // TMPBus errors
    BPNG_TMPBUS_COPYRDB_ERROR = 17,  ///< Failed to copy the reference data base to the logger's tmp directory
    BPNG_TMPBUS_SEND_MSG_ERROR = 18, ///< Failed to send a TMP bus request message
    BPNG_TMPBUS_REQUEST_ERROR = 19,  ///< The TMP bus request execution failed

    // file handling error
    BPNG_FAILED_TO_CREATE_LOCAL_FILE_OR_DIRECTORY = 20, ///< Failed to create local file or directory
    BPNG_LOCAL_PATH_NOT_FOUND = 21,   ///< Local path not found
    BPNG_READ_LOCAL_FILE_ERROR = 22,  ///< Failed to read local file
    BPNG_WRITE_LOCAL_FILE_ERROR = 23, ///< Failed to write local file
    BPNG_FILE_EXISTS_ERROR = 24,      ///< Local file already exists
    BPNG_DIR_EXISTS_ERROR = 25,       ///< Local directory already exists
    BPNG_TARGET_PATH_TOO_LONG = 26,   ///< Specified path exceeds the max. valid length (e.g. 260 for Windows systems)
    BPNG_ZIP_EXCEEDS_FATFS_MAX = 27,  ///< ZIP file exceeds max size for FAT32 file systems

    // xml error
    BPNG_XML_PARSER_ERROR = 28, ///< Error while parsing xml file

    // user error
    BPNG_INITIALISATION_ERROR = 29, /**< BPNGClient instance is not initialised or with the wrong function.
				    Use IBPNGClient::initOnline for data download or conversion directly from the device and
									IBPNGclient::iniOffline for data conversion from an offline data set.*/

    // SQLITE error
    BPNG_RDB_SQLITE_QUERY_ERROR = 30, ///< Error when trying to read data from the rdb.
    BPNG_RDB_OPEN_FAILED = 31,        ///< Failed to open the reference data base.

    // conversion error
    BPNG_CONVERSION_ERRORS = 32,        /**< Multiple conversion errors. Use IBPNGClient::getNumConversionErrors()
	                                    and IBPNGClient::getConversionError() for further information*/
    BPNG_CONV_SET_NOT_FOUND = 33,       ///< The passed conversion set pointer was not created with this IBPNGClient instance and dus could not be found.
    BPNG_NOTHING_TO_CONVERT = 34,       ///< There is no data available that could be converted. Check the specified time/id spans
    BPNG_TMT_FILE_ID_ERROR = 35,        ///< Invalid TMT/XTMT file id while trying to convert data.
    BPNG_TMT_FORMAT_ERROR_VERSION = 36, ///< The TMT/XTMT version of the trace file is not supported by this lib version.
    BPNG_TMT_FORMAT_ERROR_TS = 37,      ///< Missing FileTimeMessage in header of TMT/XTMT file
    BPNG_INVALID_MESSAGE_ERROR = 38,    ///< Invalid messages found in trace file(s).
    BPNG_INVALID_MESSAGE_ID = 39,       ///< Invalid message id found in trace file(s).
    BPNG_INVALID_MESSAGE_TS = 40,       ///< Invalid message ts found in trace file(s).
    BPNG_INVALID_MESSAGE_SUBID = 41,    ///< Invalid message sub id found in trace file(s).
    BPNG_INVALID_MESSAGE_LEN = 42,      ///< Invalid message length found in trace file(s).
    BPNG_CONV_FORMAT_ERROR = 43,        ///< Invalid format assignment or mismatching recorded trace data for the specified conversion format.

    // download error
    BPNG_DOWNLOAD_ERRORS = 44,     /**< Multiple download errors. Use IBPNGClient::getNumDownloadErrors()
	                               and IBPNGClient::getDownloadError() for further information*/
    BPNG_NOTHING_TO_DOWNLOAD = 45, ///< There is no data available that could be downoaded. Check the specified time/id spans

    // various
    BPNG_INVALID_OFFLINE_SET = 46,    ///< Failed to initialise the IBPNGClient from the passed offline data set
    BPNG_PARAMETER_MISMATCH = 47,     ///< currently not used
    BPNG_FW_VERSION_CHECK_ERROR = 48, ///< The verification of the new firmware at the end of a firmware update failed.
    BPNG_USER_CANCELLED = 49,         ///< currently not used
    BPNG_MIN_VERSION_ERROR = 50,      ///< The current library version does not suffice the the reuired min version written to BPNGError::msg
    BPNG_EXCEPTION = 51,              ///< Some kind of unhandled exception was thrown.
    BPNG_INCOMPATIBLE_RDB = 52,       ///< The logger's or offline data set's RDB-Verison is incompatible to this library version
    BPNG_UNSPECIFIED_ERROR = 53,      ///< An unspecified error occurred

    // CCP XCP errors
    BPNG_CCP_XCP_PARSER_ERROR = 54,
    BPNG_CCP_XCP_DBC_GENERATOR_ERROR = 55,
    BPNG_CCP_XCP_SEQUENCE_GENERATOR_ERROR = 56,

    BPNG_INSUFFICIENT_DISK_SPACE = 57,
    BPNG_FWUPDATE_FAILED = 58,
    BPNG_INDEX_OUT_OF_RANGE_ERR = 59,
    BPNG_READ_CONFIG_BACKUP_ERR = 60,

    //TSL
    BPNG_INVALID_RPC_COMMAND = 61,          ///< if a rpc command for tsl is wrong
    BPNG_INVALID_TSL_CASCDING = 62,         ///< if cascading of tsl is invalid

    //PWd   
    BPNG_LOGIN_CANCELED = 63,
    BPNG_USER_PWD_WRONG = 64,
    BPNG_NO_ACCESS_FOR_FUNCTION = 65,
    
    // streaming error
    BPNG_STREAMING_PROTOCOLL_ERROR = 68,
    BPNG_STREAMING_SOCKET_ERROR = 69,
    BPNG_STREAMING_DISABLED = 70,
    BPNG_FW_DEPRECATED = 71,
    BPNG_STREAMING_ABORTED_BY_PEER = 72,

    //TSL
    BPNG_INCONSISTENT_TSL_FWVERSIONS = 80   ///< if fw versions on tsl clusters are inconsistent

    // Highest enum value is BPNG_INCONSISTENT_TSL_FWVERSIONS = 80
};

enum FWUpdateErrorCode
{
    FWUPDATE_ERRORCODE_NO_ERR = 0,
    FWUPDATE_ERRORCODE_FW_PKT_NAME_EMPTY = -2,
    FWUPDATE_ERRORCODE_FW_PKT_MISSING = -3,
    FWUPDATE_ERRORCODE_NAMED_PIPE_SERVER_MKNOD = -4,
    FWUPDATE_ERRORCODE_NAMED_PIPE_SERVER_OPEN = -5,
    FWUPDATE_ERRORCODE_FW_UPDATE_NOT_IN_PROGRESS_TIMEOUT = -6,
    FWUPDATE_ERRORCODE_MISSING_LINUX_DISTR = -7,
    FWUPDATE_ERRORCODE_MISSING_LIBTMLIB_FILE = -8,
    FWUPDATE_ERRORCODE_MISSING_TMLIB_FILE = -9,
    FWUPDATE_ERRORCODE_MISSING_ATOM_FILE = -10,
    FWUPDATE_ERRORCODE_MISSING_CLIENT_FILE = -11,
    FWUPDATE_ERRORCODE_MISSING_FPGAA_FILE = -12,
    FWUPDATE_ERRORCODE_MISSING_FPGAB_FILE = -13,
    FWUPDATE_ERRORCODE_MISSING_EXTENSION_BOARD_FPGA_FILE = -14,
    FWUPDATE_ERRORCODE_MISSING_GBE_FILE = -15,
    FWUPDATE_ERRORCODE_MISSING_SBC_FILE = -16,
    FWUPDATE_ERRORCODE_MISSING_SBC_FLASH_SCRIPT = -17,
    FWUPDATE_ERRORCODE_MISSING_FPGA_FLASH_SCRIPT = -18,
    FWUPDATE_ERRORCODE_MISSING_RCV_FILE = -19,
    FWUPDATE_ERRORCODE_MISSING_LINUX_SETUP_ARCHIVE = -20,
    FWUPDATE_ERRORCODE_UNKNOWN_MB_HW_VERSION = -21,
    FWUPDATE_ERRORCODE_UNKNOWN_EXTENSION_BOARD = -22,
    FWUPDATE_ERRORCODE_UNKNOWN_EXTENSION_BOARD_VARIANCE = -23,
    FWUPDATE_ERRORCODE_NOT_READABLE_EXTENSION_BOARD_VARIANCE = -24,
    FWUPDATE_ERRORCODE_NOT_READABLE_EXTENSION_BOARD_HW_VERSION = -25,
    FWUPDATE_ERRORCODE_NOT_READABLE_HW_TYPE_VERSION = -26,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_APP_LIBS = -27,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_RC = -28,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_GBEC = -29,
    FWUPDATE_ERRORCODE_CONV_CFG_ERROR = -30,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_LINUX_KERNEL = -31,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_LINUX_KERNEL_MODULES = -32,
    FWUPDATE_ERRORCODE_FAILED_CPY_LINUX_KERNEL = -33,
    FWUPDATE_ERRORCODE_FAILED_UNCOMPRESS_CLIENT_FILE = -34,
    FWUPDATE_ERRORCODE_FAILED_CPY_CLIENT_FILE = -35,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_LINUX_DISTR = -36,
    FWUPDATE_ERRORCODE_FAILED_SBC_FLASH = -37,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_CCP_XCP = -38,
    FWUPDATE_ERRORCODE_FAILED_UPDATE_CCP_XCP_SEED_KEY_SERVERS = -39,
    FWUPDATE_ERRORCODE_MISSING_CCP_XCP_FILE = -40,
    FWUPDATE_ERRORCODE_MISSING_CCP_XCP_SEED_KEY_SERVER_FILE = -41,
    FWUPDATE_ERRORCODE_MISSING_SPYNIC_FILE = -42,
    FWUPDATE_ERRORCODE_MISSING_LOADING_ISPVM = -43,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FPGAB_FILE = -44,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FPGAA_FILE = -45,
    FWUPDATE_ERRORCODE_UNREADY_FPGAA = -46,
    FWUPDATE_ERRORCODE_LINUX_KERNEL_MAY_FREEZE_SYSTEM = -47,
    FWUPDATE_ERRORCODE_NOT_SET_DEVICE_PATH = -48,
    FWUPDATE_ERRORCODE_NOT_SET_FW_FILE = -49,
    FWUPDATE_ERRORCODE_NOT_SET_FPGA_KEY = -50,
    FWUPDATE_ERRORCODE_MISSING_DEVICE_FILE = -51,
    FWUPDATE_ERRORCODE_MISSING_FPGA_FW_FILE = -52,
    FWUPDATE_ERRORCODE_MISSING_TMUDEVQ = -53,
    FWUPDATE_ERRORCODE_UNKNOWN_DEVICE_PATH = -54,
    FWUPDATE_ERRORCODE_FAILED_CPY_FPGA_FILE = -55,
    FWUPDATE_ERRORCODE_FAILED_LINKING_FW_UPDATE = -56,
    FWUPDATE_ERRORCODE_ERROR_FLASH_FPGA = -57,
    FWUPDATE_ERRORCODE_ERROR_FLASH_SPYNIC = -58,
    FWUPDATE_ERRORCODE_ERROR_LOADING_FPGA_JTAG_DRIVER = -59,
    FWUPDATE_ERRORCODE_MISSING_EXTENSION_BOARD_VIA_PCIE = -60,
    FWUPDATE_ERRORCODE_FAILED_CONV_CFG = -61,
    FWUPDATE_ERRORCODE_FAILED_UPLOAD = -62,
    FWUPDATE_ERRORCODE_FWUFOLDER_EXISTS = 63,
    FWUPDATE_ERRORCODE_UNDEFINED = -1
};

/// Warning codes
/**
  Warnings are notified by listener calls to the function IBPNGClientListener::onWarning()
*/
enum BPNGWarningCode
{
    BPNG_NOWARNING, /// no warning
    BPNG_WARNING_CLOSE_TRACE_FILES, ///< Failed to close the current trace files on the logger device when trying to execute IBPNGClient::initOnline()
    BPNG_WARNING_MESSAGES_NOT_CONVERTED ///< In case of protocol mismatch between recorded data and target format or unsupported message sub types, it is possible that some messages can not be converted to the selected format.
};

/// Languages
/**
  ID for specifiying the language in that the library handles process and error information.
  Default language is english.
*/
enum LanguageID
{
    BPNG_GERMAN, ///< english
    BPNG_ENGLISH ///< german
};

// ===========================================================================
/// Mode for the IBPNG::downloadBugReport() function
enum BPNGBugreportMode
{
    BR_FULL_WO_TRACES = 0,      ///< Full bug report without traces
    BR_ONLY_LOGS = 1,           ///< Only log files are downloaded
    BR_FDB_RDB = 2,             ///< only FDB and RDB are downloaded
    BR_ONLY_CLIENT = 3,         ///< only client logs are stored
    BR_FULL_ALL_TRACES = 4,     ///< Full bug report with all traces files
    BR_FULL_TIMESPAN_TRACES = 5 ///< Full bugreport with trace file of a specified time span
};

// ===========================================================================
/// Currently supported interfaces
enum ChannelType
{
    CH_UNDEFINED = 0, ///< undefined channel type
    OBSOLETE_CH_CANLS,///< CAN low speed interface
    CH_CAN,           ///< CAN high speed interface
    CH_LIN,           ///< LIN interface
    CH_SERIAL,        ///< Serial interface
    CH_ETHERNET,      ///< Ethernet interface
    CH_FLEXRAY,       ///< Flexray interface
    CH_MOST25_CTRL,   ///< MOST 25 control channel
    CH_MOST25_MDP,    ///< MOST 25 data packet channel (MDP)
    CH_MOST25_SYNC,   ///< MOST 25 synchroneous channel (streaming data)
    CH_MOST150_CTRL,  ///< MOST 150 control channel
    CH_MOST150_MDP,   ///< MOST 150 data packet channel (MDP)
    CH_MOST150_MEP,   ///< MOST 150 ethernet packet channel (MEP)
    CH_MOST150_STREAM,   ///< MOST 150 synchroneous channel (streaming data)
    CH_ANALOG_IN,     ///< Analog in
    CH_DIGITAL_IN,    ///< Digital in
    CH_CAMERA,        ///< Camera channel
    CH_CCPXCP,        ///< CCP XCP
    CH_DIAG,          ///< Diagnose, currently not used
    CH_GPS,           ///< Global Positioning System
    CH_ECL,           ///< Electronic Control Line
    CH_COMPLEXFILTER  ///< Electronic Control Line  
};


// ===========================================================================
/// Identifier for currently supported formats
enum FormatId
{
    UNDEFINED = 0,
    TMASC = 1,		    ///< Telemotive ASCII format
    OP2 = 2,		    ///< MOST OP" format
    CANOE = 3,		    ///< CANOE ASCII format
    STA = 4,		    ///< Serial Trace Analyser format
    GNLOG = 5,		    ///< GNLog format
    TCLOG = 6,		    ///< Trace Client format
    TCLOG_TS = 7,	    ///< Trace Client format plus time stamps
    RAW_SERIAL = 8,	    ///< Raw Serial Format
    RESERVED_1 = 9,	    ///< Reserved, formerly xml
    IMG = 10,		    ///< DataAnalyser IMG format
    TMBIN = 11,		    ///< Telemotive binary format
    CANCORDER = 12,	    ///< CANCorder format
    APN = 13,		    ///< APN serial format
    ASCHEX = 14,	    ///< ASCII format that writes binary data as hex values
    WAV = 15,		    ///< Wave PCM Format, use this for MOST Sync
    TCPDUMP = 16,	    ///< TCP dump (*.pcap) format
    BLF = 17,		    ///< Binary Logging Format - Vector Informatik
    DLT_BMW = 18,	    ///< BMW specific DLT Format
    MDF = 19,		    ///< Measurement data format - Vector Informatik
    MDF_CAN_SIG = 20,	    ///< MDF format based on CAN signals
    MPEG4_BLOCKS = 21,	    ///< Camera format seperated blocks
    MPEG4_JOINED = 22,	    ///< Camera joined videos
    NMEA = 23,		    ///< Raw format for GPS data
    KML = 24,		    ///< KML format for GPS data
    KMZ = 25,		    ///< KMZ format for GPS data
    SERIAL_DEBUG = 26,	    ///< Serial Format with "new Line"-packaging
    RAW_ETHERNET = 27,	    ///< Raw ethernet format
    ESO_TRACE = 28,	    ///< Audi specific ESO Format
    XTMT = 29,		    ///< Extended Telemotive Trace File format
    GPX = 30,		    ///< GPS Exchange format
    MDF_CCP_XCP_SIG = 31,   ///< MDF format based on CCP & XCP signals.
    TS = 32,                ///< Raw isochrone format
    FILTER_FORMAT = 33,     ///< Filter format= no change in format
    INVALID = 0xFF
};


// ===========================================================================
///
enum PwdPrivilegesFuncId
     {
        REMOVE_DATA = 0,
        SET_TIME,
        SET_EVENT,
        RECONFIG,
        RECONFIG_PASSWORD,
        RECONFIG_COMPLEX_FILTER,
        UPLOAD_WINE_DLLS,
        UPDATE_FIRMWARE,
        CHANGE_LICENCES,

        // must be last enum value
        PRIVILEGES_END
    };


// ===========================================================================
// List of possible reasons why a measure signal was ignored
enum Reason
{
    R_UNSUPPORTED_BIT_MASK,     ///< DBC file don't support bit operations with a bit mask.
    R_BIT_MASK_OVERLAP,         ///< Bit mask is incorrect and cause a overlap with at least one other signal.
    R_UNSUPPORTED_COMPU_TAB,    ///< DBC file don't support all compu tab types; only tab will ignored, not the signal itself!
    R_FORBIDDEN_TAB_VALUE,      ///< DBC file don't support all possible values of a compu tab; only tab will ignored, not the signal itself!
    R_UNKNOWN                   ///< Unknown reason.
};

// ===========================================================================
/// False measure signal interface
struct IFalseMeasureSignal
{
    /// Returns the device Id
    virtual uint8_t getDeviceId() const = 0;
    /// Returns the signal number
    virtual uint16_t getSignalNo() const = 0;
    /// Returns the ignore reason
    virtual Reason getIgnoreReason() const = 0;
};

// ===========================================================================
/// False measure signal list interface
struct IFalseMeasureSignalList
{
    /// Returns the number of signals
    virtual size_t getSize() const = 0 ;
    /// Returns the IFalseMeasureSignal at index
    virtual const IFalseMeasureSignal* getSignal(size_t index) const = 0;
};

// ===========================================================================
/// Channel interface
struct IChannel
{
    /// Returns the ChannelType
    virtual ChannelType getType() const = 0;
    /// Returns the channel's index
    virtual uint8_t getIndex() const = 0;
    /// Returns the channel's name
    virtual const char* getName() const = 0;
    /// Returns the channel's package id
    virtual uint32_t getMainboardNumber() const = 0;
    /// Returns the channel's offset
    virtual uint32_t getOffset() const = 0;
};

// ===========================================================================
/// Channel interface
struct ITesttoolsChannel
{
    /// Returns the IChannel of this ITesttoolsChannel
    virtual IChannel* getIChannel() const = 0;
    /// Returns whether the channel matches with the contained channel
    virtual BOOL matchIChannel(const IChannel* iChannel) const = 0;
    /// Returns the channel's containerId
    virtual uint32_t getContainerId() const = 0;
    /// Returns the channel's associated containerId
    virtual uint32_t getPseudoContainerId() const = 0;
    /// Returns the channel's associated containerId name
    virtual const char* getPseudoChannelName() const = 0;
    /// Returns the channel's containerId
    virtual uint16_t getBaseCanId() const = 0;
    /// Returns the channel's containerId is extended or not
    virtual bool isExtendedCanId() const = 0;
    /// Returns the ethernet host ip
    virtual const char* getHostIp() const = 0;
    /// Returns the ethernet device ip
    virtual const char* getDeviceIp() const = 0;
    /// Returns the ethernet device ip
    virtual unsigned int getDevicePort() const = 0;
    /// Returns protocol
    virtual int getProtocol() const = 0;
    /// Returns debuglevel
    virtual int getDebugLevel() const = 0;
};

// ===========================================================================
/// Channel list interface
struct IChannelList
{
    /// Returns the number of channels
    virtual uint8_t getSize() const = 0 ;
    /// Returns the IChannel at index
    virtual const IChannel* getChannel(uint8_t index) const = 0;
};

// ===========================================================================
/// TesttoolsChannel list interface
struct ITesttoolsChannelList
{
    /// Returns the number of channels
    virtual uint8_t getSize() const = 0 ;
    /// Returns the ITesttoolsChannel at index
    virtual const ITesttoolsChannel* getTesttoolsChannel(uint8_t index) const = 0;
};


// ===========================================================================
/// FormatInfo interface
struct IFormatInfo
{
    /// Returns the FormatId
    virtual FormatId getFormatId() const = 0;

    /// Returns the format's name
    virtual const char* getName() const = 0;

    /// Returns whether the format supports multiple channels in one output file
    virtual BOOL isMultipleChannelSupport() const = 0;

    /// Returns whether the format is binary
    // 0 = false, else = true
    virtual BOOL isBinaryFormat() const = 0;

    /// Returns the format's default extension
    virtual const char* getExtension() const = 0;

    /// Returns the number of supported channel types
    virtual int getNumSupportedChannelTypes() const = 0;

    /// Returns one supported ChannelType
    virtual ChannelType getChannelType(int index) const = 0;

    /// Returns the required license for the format, an empty string for free formats
    virtual const char* getRequiredLicense() const = 0;
};

// ===========================================================================
/// Format list interface
struct IFormatList
{
    /// Returns the number of available formats
    virtual int getSize() const = 0;

    /// Returns the IFormat at index
    virtual const IFormatInfo* getFormatInfo(int index) const = 0;
};

// ===========================================================================
// Conversion set interface
/// A conversion set stores all conversion relevant settings
/**
  To convert trace data a conversion set must be created.
  Several channels can be added to one conversion set. The trace data
  of that channels are converted to the assigned formats.
  The conversion set also includes the data spans that has to be converted.
*/
struct IConversionSet
{
    /// Adds a channel to the conversion set and assigns the target format to it.
    /**
      Use the IBPNGClient::getLoggerChannel() function to get all existing channels.

      @param channelType must be one of the appropriate ChannelType enum.
      @param channelIndex zero-based channel index
      @param formatId must be one of the appropriate FormatId enum.
      @param fileId The data of all channels with same formatId and same fileId are written to the same output file. The default value -1 indicates always a separate file for each channel.
    */
    virtual void addChannel(ChannelType channelType, uint8_t channelIndex, FormatId formatId, int fileId = -1, int offset = 0, int mbnr = -1) = 0;

    /// Adds a time span to the conversion set.
    /**
      The data within the time span will be converted to the specified formats.
      @param startTime must be in usec since 01.01.1970 (UTC)
      @param endTime must be in usec since 01.01.1970 (UTC)
    */
    virtual void addTimeSpan(uint64_t startTime, uint64_t endTime) = 0;

    /// Adds a ReferenceDB ID range to the conversion set.
    /**
       Passed parameter are IDs from the Reference Data Base (RDB). After calling
       on of the init functions IBPNGClient::initOnline() or IBPNGClient::initOffline()
       you can get the path to the RDB with IBPNGClient::getReferenceDataBasePath().

       The RDB includes all occurred events like startups, shutdowns, etc. but also
       all recorded trace files. Each RDB entry has a unique DataBaseEntryID.
       With this function you can easily select data between arbitrary RDB entries.
       For example you can convert all data between index X (which is e.g. a startup)
       and index Y (which is e.g. a shutdown). When the DataBaseEntryId of a trace
       file is passed, this trace block will be included by the conversion.

       @param startId DataBaseEntryId that inicates the start of the data range to be converted
       @param endId DataBaseEntryId that inicates the end of the data range to be converted
    */
    virtual void addRdbIdRange(uint64_t startId, uint64_t endId) = 0;

    /// Loads the format settings from an ini file
    virtual bool loadFormats(const char* pathToIniFile) = 0;

    /// Saves the format settings to an ini file
    virtual bool saveFormats(const char* pathToIniFile) = 0;
};

enum RdbEventType
{
    UNKNOWN = 0,
    STARTUP =0x01,              //!< bp2 startup
    SHUTDOWN = 0x02,            //!< bp2 shutdown
    MARKER = 0x03,              //!< Marker set
    INFO = 0x05,                //!< Info is set
    SLAVE_OFFSET = 0x06,        //!< cascading slave offset
    SLAVE_TO_MASTER = 0x07,     //!< cascading slave to master
    DATA_DELETED = 0x08,        //!< All data and data space is deleted
    TIME_SET =0x09,             //!< bp2 time was set
    NEW_TIME = 0x0A,            //!< new time
    SUDDEN_DEATH = 0x0B,        //!< no "real" shutdown was found after startup.
};

/// Interface to an RDB event
struct IRdbEvent
{
    /// Get type of event.
    virtual RdbEventType WINAPI getType() = 0;

    /// Returns the unique entry ID that can be set to DataSpans for data download and conversion
    virtual uint64_t WINAPI getUniqueId() = 0;

    /// Returns the event's time stamp in usec since 01.01.1970 UTC
    virtual uint64_t WINAPI getTimeStamp() = 0;

    /// Returns the logger's time zone that was active at the event's time stamp
    virtual const char* WINAPI getTimeZone() = 0;

    /// Returns the index of this event. Only used for marker events
    virtual int WINAPI getIndex() = 0;

    /// Returns additional information. The meaning of this string depends on the event's type. See RDB specification document for more information.
    virtual const char* WINAPI getComment() = 0;
};

/// Interface to a list of rdb events
struct IRdbEventList
{
    /// Returns the size of the event list
    virtual int WINAPI getSize() = 0;

    /// Returns a pointer to the IRdbEvent at index
    virtual IRdbEvent* WINAPI getEvent(int index) = 0;
};


/// Logger status
enum BPNGLoggerStatus
{
    LS_OK = 0,     // Device is ok
    LS_ERROR = 1,  // Device has at least one active error
    LS_NOSYNC = 2, // Device is configured as slave but no master is found
    LS_WARNING = 3,
    LS_FWUPDATE = 4,
    LS_UNDEFINED = -1
};


// ===========================================================================
/// Struct with information about a logger found in LAN
struct OnlineLoggerInfo
{
    const char* ip;             ///< the logger's ip address
    const char* name;           ///< the logger's name
    const char* mbnr;           ///< mainboard number
    uint8_t connected;          ///< connected with client, 0=false, else true
    const char* currentUser;    ///< user name of connected pc account
    uint8_t loggerStatus;       ///< current logger status, @sa BPNGLoggerStatus
    uint8_t wlan;               ///< Flag for connection type. 0 = ehternet, 1 = wlan
    const char* tslEth0IP;      ///< ip address of device connected to eth0, 0.0.0.0 if none
    const char* tslEth1IP;      ///< ip address of device connected to eth1, 0.0.0.0 if none
    int8_t tslId;               ///< id for device in tsl network, continues in tsl, starts with 0 on first device
    int32_t tslNetworkId;       ///< id of tsl network, -1 = no TSL
    const char* tslName;        ///< name(id) of tsl network
};

// ===========================================================================
/// Types for DataSpan
enum DataSpanType
{
    DST_IDSPAN = 0,
    DST_TIMESPAN = 1
};

// ===========================================================================
///
struct DataSpan
{
    uint8_t type;   ///< set type to 0 for a id based range, set type to 1 for a time based range
    uint64_t start; ///< start time/id of data span
    uint64_t end;   ///<  end time/id of data span
};

// ===========================================================================
/// Error struct with error code and optional error message
struct BPNGError
{
    BPNGErrCode code; ///< error code
    const char* msg;  ///< error message
};


// ===========================================================================
/// 
struct LogInData
{
    const char* userName;
    const char* userPwd;  
};

// ===========================================================================
/// Common properties, deprecated use \a ICommonProperties interface instead
struct CommonProperties
{
    char* nameOfTester;
    uint32_t maxOutputSizeMB;
    uint8_t separatedTimeFormat;

    uint8_t separatedTimeFormatInOfflineSet;

    char* alternativeLoggerName;
    uint8_t useAlternativeLoggerName;
    uint8_t useSubdirectories;
    uint8_t midnightSplitting;
    uint8_t fileTimeSpansLikeSelection;

    uint8_t markerNumbersInFileNames;
    uint8_t subfolderWithLoggerName;

    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// CAN pseudo message properties, deprecated use \a ICANPseudoProperties interface instead
struct CANPseudoMessagesProperties
{
    uint8_t canTimeStampMessage;
    uint32_t channelTS;
    uint32_t dlcTS;
    uint32_t canIdTS;
    uint32_t hourBitPos;
    uint32_t minuteBitPos;
    uint32_t secondBitPos;
    uint32_t dayBitPos;
    uint32_t monthBitPos;
    uint32_t yearBitPos;
    uint8_t canTriggerMessage;
    uint32_t channelTrigger;
    uint32_t dlcTrigger;
    uint32_t canIdTrigger;
    uint32_t triggerNumBitPos;
    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// MOST pseudo properties, deprecated use \a IClientProperties interface instead
struct MOSTPseudoMessagesProperties
{
    uint8_t writeMessages;  ///< Active flag for writing MOST pseudo messages for trigger
    uint32_t source;          ///< Source address
    uint32_t target;          ///< Target address
    uint32_t functionBlockID; ///< Function block ID
    uint32_t functionID;      ///< Function ID
    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint32_t reserved4;
};

/// Conversion Properties, deprecated use \a IClientProperties interface instead
struct ConversionProperties
{
    uint8_t useSatelliteTimeForGPSFormats;
    char* isochronalMost150Channels;
};

/// ClientProperties, deprecated use \a IClientProperties interface instead
struct ClientProperties
{
    CommonProperties common;
    CANPseudoMessagesProperties canPseudoMessages;
    MOSTPseudoMessagesProperties mostPseudoMessages;
    ConversionProperties conversionProp;
};

/// Pointer to a function named onLogRequest with one parameter and no return value
typedef void (WINAPI*onLogRequest)(const char* logRecord);

#endif // __BPNGDEFINES_H__
